(*
   Here are three functions, all called outer,
   each with the same function called inner nested
   within them. The first example calls the inner
   function. The second example passes the inner
   function as a parameter to another function.
   And the third example uses the inner function
   as a return value.
*)

fun outer x y =
  let
    fun inner y = x + y  (* x is a "non local" reference *)
  in
    inner y              (* call the inner function *)
  end;


fun outer x aList =
  let
    fun inner y = x + y  (* x is a "non local" reference *)
  in
    map inner aList      (* pass the inner function as an arg to map *)
  end;


fun outer x =
  let
    fun inner y = x + y  (* x is a "non local" reference *)
  in
    inner                (* return the inner function *)
  end;


(* Some examples of using this last definition of outer. *)
val f = outer(3);
f(5);
f(4);
f(~2);
val g = outer(100);
g(2);
g(20);


(* Here is the same example but using an anonymous inner function. *)
fun outer2 x = (fn y => x + y);

val h = outer2(1000);
h(50);
h(5);


(* Here is the same example again, written as a curried function.
   Notice that it no longer looks like an example of a "nested function",
   but it is. *)
val outer3 = fn x => fn y => x+y;


(* In the textbook on page 201, this example is written the following way. *)
fun funToAddX x =
  let
    fun addX y =
      y + x
    in
      addX
    end;